﻿Type=StaticCode
Version=4.3
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
 Private Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

	'Code written by Johan Schoeman - April/May 2014 (Durban, South Africa)
	'email: johans@astrapak.com / johan@barrierfilm.co.za
	'cell: +27 82 783 6322
	'This code will draw layer 1 to layer 11 full Aztec codes
	'It will allow you to enter up to approximately 500 characters (typed as words with spaces, etc) or about 750 numbers
	'Up to layer 11 should be sufficient to encode the majority of information that
	'one might want to encode (such as for eg Aztec codes on airline boarding passes)
	'Aztec codes seem to be slightly more complicated than QR codes especially as far as the packing of the Aztec code is concerned.
	'Packing up to layer 11 is not complicated but from layer 12 onwards it gets a bit tricky
	'It is important to note that the timing patterns must always be packed in such a way that there is a black pixel
	'in line with the centre pixel of the finding pattern. Up to layer 11 the timing pattern has black pixels
	'on the extreme outer edges but from layer 12 (at 32 pixels from the centre) it changes so that there is a white pixel
	'on the outer edges. This will once again revert to a black pixels on the outer edges as soon as the next timing pattern
	'needs to kick in which should be at 48 pixels from the centre (i.e at about layer 20 and then again 16 pixels further out
	'at layer 28)

	Dim Rect1 As Rect    
	Dim sf As StringFunctions            'a library by Margret - thanks Margret!
	Dim mat(65, 65) As Int               'The matrix that will be used to store all 1's and 0's of the Aztec code before it is drawn
	Dim lengte_incl_err_words, layer, siz, cap, use_bits, stuff_tel As Int
    Dim bcolor As Int
	Dim fcolor As Int
	Dim shp As String
	
End Sub
'Description: Generates an Aztec code and saves it in the /pictures folder as "Aztec.png" from where it can be loaded into for eg an ImageView
'
'Method of calling:
'AztecCode.DrawAztecCode(message,back_color,fore_color,shape)
'
'where 'message' = message to be encoded - type String
'where 'back_color' = background color of the QR Code - type Int
'where 'fore_color' = foreground color of the QR Code (pixel color) - type Int
'where 'shape' = the shape of the pixels (c or C = circle, s or S = square) - type string
'example of parameters to pass:
'message = "Hallo, how are you today"
'back_color = Colors.Yellow
'fore_color = Colors.Black
'shape = "c"
Public Sub DrawAztecCode (mes As String, bc As Int, fc As Int, shape As String)

Dim main_message,karakter,bin_str As String
Dim lengte,teller,wyser As Int

shp = shape
bcolor = bc
fcolor = fc
sf.Initialize()
main_message = mes                       'Read the input message to be incoded from Sheet 1 Cell A1
lengte = sf.Len(main_message)                       'Get the length of the input message (i.e number of characters)

Dim what_table(1400) As Int                         'This will keep track of what table the chars of the input message can be found in (i.e 1 to 5)
Dim char_value(1400) As Int                         'This will keep the values of the chars in the input message (i.e 0 to 31)
Dim new_char_value(1400) As Int                     'This will keep the entire input message (decimal) including jumps between the tables

'The following is the standard Aztec code lookup tables (i.e 5 tables)
'Chars in Tables 1 to 4 (upperc, lowerc, mixed, punc) will be encoded into 5 bit binary values while chars in Table 5 (the DIGIT table) will be encoded into 4 bit binary values
'binary value '01110' in Table 5 is encoded as 1110 (i.e the leading 0 is dropped) = Upper Latch
'Binary value '01110' in Table 1  = M
'Binary value '01110' in Table 2  = m
'Binary value '01110' in Table 3  = Chr(13) = Carriage Return
'Binary value '01110' in Table 4  = )

Dim upperc(),lowerc(),mixed(),punc(), digit() As String
'Upper Case
upperc = Array As String("P/S", " ", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "L/L", "M/L", "D/L", "B/S")
'Lower Case
lowerc = Array As String("P/S", " ", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "U/S", "M/L", "D/L", "B/S")
'Mixed
mixed = Array As String("P/S", " ", Chr(1), Chr(2), Chr(3), Chr(4), Chr(5), Chr(6), Chr(7), Chr(8), Chr(9), Chr(10), Chr(11), Chr(12), Chr(13), Chr(27), Chr(28), Chr(29), Chr(30), Chr(31), "@", "\", "^", "_", "`", "|", "~", Chr(127), "L/L", "U/L", "P/L", "B/S")
'Punctuation
punc = Array As String("FLG(n)", Chr(13), Chr(13) & Chr(10), ". ", ", ", ": ", "!", """", "#", "$", "%", "&", "'", "(", ")", "*", "+", ",", "-", ".", "/", ":", ";", "<", "=", ">", "?", "[", "]", "{", "}", "U/L")
'Digit
digit = Array As String("P/S", " ", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ",", ".", "U/L", "U/S", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "")


'Now let's find what table (Upp, Low, Mix, Punct, Digit) each input char appears in and then also what the index (0 to 31) of the char in that table is
teller = 0
Do While teller <> lengte
  karakter = sf.Mid(main_message, teller + 1, 1)      'get the character from the string - 1 char at a time
  For i = 0 To 31
    If karakter = upperc(i) Then
      what_table(teller) = 1                          'the char appears in table 1
      char_value(teller) = i                          'It is the i'th (0...31) char in that table
      Exit
    Else If karakter = lowerc(i) Then
      what_table(teller) = 2                          'the char appears in table 2
      char_value(teller) = i
      Exit
    Else If karakter = mixed(i) Then
      what_table(teller) = 3                          'the char appears in table 3
      char_value(teller) = i
      Exit
    Else If karakter = punc(i) Then
      what_table(teller) = 4                          'the char appears in table 4
      char_value(teller) = i
      Exit
    Else If karakter = digit(i) Then
      what_table(teller) = 5                          'the char appears in table 5
      char_value(teller) = i
      Exit
    End If
  Next 
  teller = teller + 1
Loop 

'This code will eliminate a few unnecessary jumps as a result of spaces in the input message and replace them with Shift rather than Latch operations
For i = 2 To lengte
  If what_table(i - 1) = 1 AND char_value(i - 1) = 1 AND what_table(i - 2) = 2 Then
    what_table(i - 1) = 2
  End If
  If what_table(i - 1) = 1 AND char_value(i - 1) = 1 AND what_table(i - 2) = 5 Then
    what_table(i - 1) = 5
  End If
  If what_table(i - 1) = 1 AND char_value(i - 1) = 1 AND what_table(i - 2) = 3 Then
    what_table(i - 1) = 3
  End If  
  
Next

teller = teller - 1

wyser = 0

'Now we are going to build a string of binary data based on the info established above. We will also add the
'necessary code/words to jump between the different tables.
bin_str = ""

'By default we need to start in table UPP (as per Aztec code specification)
'We will now determine where the very first char in the input message is (i.e what table) and then add
'the necessary words to get there from table UPP. We start with the very first input char to establish if a jump(s) are required
'to a table other than UPP to encode the first char in the input message
For i = 0 To teller - 1
 If i = 0 Then
  If what_table(i) = 2 AND what_table(i + 1) = 3 Then                    'by default/specification we start at table upperc for the first char
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 2 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 2 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29    'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
   End If
   If what_table(i) = 2 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
   End If
   If what_table(i) = 2 AND what_table(i + 1) = 5 Then     
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
   End If
   
  If what_table(i) = 3 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29    'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28    'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  
    
  If what_table(i) = 4 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = 29    'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31    'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30    'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    
  End If
  
  
  If what_table(i) = 1 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  
  
  If what_table(i) = 5 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = 30 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
  End If
 End If
'***************************************************************************

'Now we determine the table, index, and jumps required for the 2nd char onwards in the input data message
 If i > 0 Then
  If what_table(i) = 1 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 1 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  
'***********************************************************************
  
  If what_table(i) = 2 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 2 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 2 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 2 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 2 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  
'***************************************************************
 
  
  If what_table(i) = 3 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
   If what_table(i) = 3 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'PUN
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 3 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  
'*************************************************************
  If what_table(i) = 4 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 4 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 31 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'DIG
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  
'****************************************************************
  If what_table(i) = 5 AND what_table(i + 1) = 5 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 1 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 2 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 28 'LOW
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 3 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
  If what_table(i) = 5 AND what_table(i + 1) = 4 Then
    new_char_value(wyser) = char_value(i)
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 14 'UPP
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)
    wyser = wyser + 1
    new_char_value(wyser) = 29 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
    new_char_value(wyser) = 30 'MIX
    bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)
    wyser = wyser + 1
  End If
 End If
Next 
 
'We need to add the last input char as the above code does not encode the last char in the input data message
new_char_value(wyser) = char_value(i)
If what_table(i) = 5 Then
  bin_str = bin_str & convert_to_binary(new_char_value(wyser), 4)     'convert it to a 4 bit long binary string
Else
  bin_str = bin_str & convert_to_binary(new_char_value(wyser), 5)     'convert it to a 5 bit long binary string
End If
'Now we have the complete input data message, including jumps between the tables, as a long binary string that are stored in bin_str

'We now build a string of the decimal values of the input data message including the jump words
'If the length of the value being tested is < 2 then we add a leading 0 to it so that values range from 00 to 31

Dim datastring, new_mess, mes, check As String
Dim lay_1_b, lay_2_b, lay_3_b, lay_4_b, lay_5_b, lay_6_b, lay_7_b, lay_8_b, lay_9_b, lay_10_b, lay_11_b As Int

Dim ncv() As String    'We need all decimal representations of input chars and established jumps to be 2 digits longs
ncv = Array As String("00","01","02","03","04","05","06","07","08","09","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31")

datastring = ""
For i = 0 To wyser
   datastring = datastring & ncv(new_char_value(i))   'build 'datastring' by making use of array 'ncv'
Next 

'Log(bin_str)
lengte = sf.Len(bin_str)
'Log(lengte)
'Let's make a best estimate of the number of bits that will be required to fill the Aztec code (data words plus error words)
'We will, as a first try, add 15% of the number of data words as error correction words to the input data
lengte_incl_err_words = Floor(1.15 * lengte)    'recommendation is to have "23% + 3 words" of data length as error correction

If lengte_incl_err_words < 3168 Then            'maximum bits that a layer 11 Aztec code can hold (data + bits stuffing + padding + error correction)
	'This is the max bits available for data and error words in an Aztec code of layers 1...11. It excludes the finding pattern, alignment marks, mode message data, and timing patterns 
	lay_1_b = 126
	lay_2_b = 288
	lay_3_b = 480
	lay_4_b = 704
	lay_5_b = 960
	lay_6_b = 1248
	lay_7_b = 1568
	lay_8_b = 1920
	lay_9_b = 2300
	lay_10_b = 2720
	lay_11_b = 3168

	'Based on the message length (incl error words), we select the required Aztec code (i.e number of layers required)
	If  lengte_incl_err_words <= lay_1_b Then
	   layer = 1                                                          'This is a 1 layer Aztec code
	   siz = 19                                                           'It will be fitted into a 19 x 19 matrix
	   cap = 6                                                            'The input bit stream will be encoded into 6-bit words
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4)                     'This Aztec code can accommodate 128 data bits (incl padding, pre-bits, error, pre-pending)
    End If                                                                'The finding pattern and mode message takes up 225 bits
	If  lengte_incl_err_words > lay_1_b AND lengte_incl_err_words <= lay_2_b Then   'We also need to subtract bits taken up by the timing patterns
	   layer = 2
	   siz = 23
	   cap = 6                                                              'The input bit stream will be encoded into 6-bit words
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4)                       '288 bits for all data
	End If
	If  lengte_incl_err_words > lay_2_b AND lengte_incl_err_words <= lay_3_b Then
	   layer = 3
	   siz = 27
	   cap = 8                                                              'The input bit stream will be encoded into 8-bit words
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4)                       '480 bits for all data
	End If
	If  lengte_incl_err_words > lay_3_b AND lengte_incl_err_words <= lay_4_b Then
	   layer = 4
	   siz = 31
	   cap = 8                                                                  'The input bit stream will be encoded into 8-bit words
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4)                           '704  bits for all data
	End If
	If  lengte_incl_err_words > lay_4_b AND lengte_incl_err_words <= lay_5_b Then
	   layer = 5
	   siz = 37
	   cap = 8
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4            '960  bits for all data
	End If                                                                       'Here we add back 4 bits where the timing patterns cross one another
	If  lengte_incl_err_words > lay_5_b AND lengte_incl_err_words <= lay_6_b Then
	   layer = 6
	   siz = 41
	   cap = 8
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4             '1248  bits for all data
	End If
	If  lengte_incl_err_words > lay_6_b AND lengte_incl_err_words <= lay_7_b Then
	   layer = 7
	   siz = 45
	   cap = 8
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4              '1568  bits for all data
	End If
	If  lengte_incl_err_words > lay_7_b AND lengte_incl_err_words <= lay_8_b Then
	   layer = 8
	   siz = 49
	   cap = 8                                                                       'The input bit stream will be encoded into 8-bit words  
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4                '1920  bits for all data
	End If
	If  lengte_incl_err_words > lay_8_b AND lengte_incl_err_words <= lay_9_b Then
	   layer = 9
	   siz = 53
	   cap = 10                                                                      'The input bit stream will be encoded into 10-bit words
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4                '2304  bits for all data
	End If
	If  lengte_incl_err_words > lay_9_b AND lengte_incl_err_words <= lay_10_b Then
	   layer = 10
	   siz = 57
	   cap = 10
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4                 '2720  bits for all data
	End If
	If  lengte_incl_err_words > lay_10_b AND lengte_incl_err_words <= lay_11_b Then
	   layer = 11
	   siz = 61
	   cap = 10
	   use_bits = (siz * siz) - 225 - (layer * 2 * 4) - (siz * 4) + 4                 '3168  bits for all data
	End If

'   Log("layer")
'	Log(layer)
'	Log("siz")
'	Log(siz)
'	Log("cap")
'	Log(cap)
'	Log("use_bits")
'	Log(use_bits)

	'Now let's do the bit stuffing!
	'As an example:
	'If the message words need to be 6 bits long (can also be 8, 10, or 12 - depending on the selected layer) then
	'the first 5 bits of each 6 bit word need to be tested to establish if they are all 0's or all 1's
	'If all 0's then the 6th bit is replaced by a 1 and the original 6th bit is shifted 1 bit to the right.
	'If all 1's then the 6th bit is replaced by a 0 and the original 6th bit is shifted 1 bit to the right.
	' 000110 000111 101001 000000 111001 will become 000110 000111 101001 000001 011100 1
	'The length of the bit stream increased by 1 bit due to bit stuffing (and will increase further if there are more occurences of
	'5 consecutive 0's or 1's in a 6 bit word
	'Once a bit has been inserted (due to bit stuffing - being it a 0 or a 1) the next 6 bits to test start at the bit that was pushed
	'to the right (i.e the bit to the right of the bit that was inserted)

	new_mess = ""                                            'We will build a new bitstream that includes bit stuffing (if any required) - new_mess
	mes = bin_str                                            'The bit stream before bit stuffing
	lengte = sf.Len(mes)
'   Log("length before stuffing")
'	Log(sf.Len(mes))
	lengte = sf.Len(mes)                                     'The lenght of the bit stream before bit stuffing
	stuff_tel = 0
	For i = 1 To lengte - (lengte Mod cap) Step cap          'Word length in number of bits (6,8,10,or 12)
	  check = sf.Mid(mes, i, cap)
	  If cap = 6 Then
	    If sf.Left(check, 5) = "00000" Then                     'For when the words are to be 6 bits long (layer 1 and 2 Aztec codes)
	      new_mess = new_mess & sf.Left(check, 5) & "1"         'first 5 bits are all 0 so insert a 1
		  i = i - 1                                             'Next 6 bits should start at the bit that was pushed to the right - so step back one bit
	      stuff_tel = stuff_tel + 1                             'Keep track of how many bits are added due to bit stuffing
	    Else If sf.Left(check, 5) = "11111" Then
	      new_mess = new_mess & sf.Left(check, 5) & "0"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else
	      new_mess = new_mess & check
	    End If
	  End If
	  If cap = 8 Then
	    If sf.Left(check, 7) = "0000000" Then                   'For when the words are to be 8 bits long (layers 3 to 8 Aztec codes)
	      new_mess = new_mess & sf.Left(check, 7) & "1"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else If sf.Left(check, 7) = "1111111" Then
	      new_mess = new_mess & sf.Left(check, 7) & "0"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else
	      new_mess = new_mess & check
	    End If
	  End If
	  If cap = 10 Then
	    If sf.Left(check, 9) = "000000000" Then                 'For when the words are to be 10 bits long (layers 9 to 22 Aztec codes)
	      new_mess = new_mess & sf.Left(check, 9) & "1"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else If sf.Left(check, 9) = "111111111" Then
	      new_mess = new_mess & sf.Left(check, 9) & "0"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else
	      new_mess = new_mess & check
	    End If
	  End If
	  If cap = 12 Then
	    If sf.Left(check, 11) = "00000000000" Then              'For when the words are to be 12 bits long (layers 23 to 32 Aztec codes)
	      new_mess = new_mess & sf.Left(check, 11) & "1"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else If sf.Left(check, 11) = "11111111111" Then
	      new_mess = new_mess & sf.Left(check, 11) & "0"
	      i = i - 1
	      stuff_tel = stuff_tel + 1
	    Else
	      new_mess = new_mess & check
	    End If
	  End If
	Next 
    
	'The new length after bit stuffing is.....
	lengte = lengte + stuff_tel                       'The length of the original bit stream + the number of bits that have been added as a result of bit stuffing
	
	'If the length of the bit stuffed message is shorter than the original message then we need to append the difference in length (from the 
	'pre-bit stuffed message to the bit stuffed message)
	If sf.Len(new_mess) < lengte Then
	  new_mess = new_mess & sf.right(mes,lengte - sf.Len(new_mess))
	End If  

'	Log("length after stuffing")
'	Log (sf.Len(new_mess))
'	Log("stuff_tel")
'	Log(stuff_tel)

    Dim a, b As String
	Dim padd, err_bits, pre_bits, no_of_data_words, no_of_error_words As Int
	a = ""
	b = ""

	bin_str = new_mess          'This is the binary string after bit stuffing....
	lengte = sf.Len(bin_str)    'and the length of the new bit stuffed string

'	Log("bin_str after stuff")
'	Log(bin_str)
'	Log(lengte)
'	Log (" ")
     
	'Now we need to establish if padding is required should the bit string not be a full multiple of the required word length (i.e 6, 8, 10, 12 bits)
	padd = lengte Mod cap                     
	If padd > 0 Then
	  padd = cap - (lengte Mod cap)               'how many padding bits need to be added
	End If
    
'	Log("padd")
'	Log (padd)
    
 
	'Now padd the bit string with the required/claculated number of 1's so that the final bit string is a multiple of full words
	If padd > 0 Then
	  For i = 1 To padd
	    bin_str = bin_str & "1"                   'Padd the message to the next full byte with 1's
	  Next 
	End If
	lengte = sf.Len(bin_str)                      'This is the length of the padded message - a multiple of cap (i.e 6, 8, 10, or 12 bits)

'   Log("bin_str after padd")
'	Log(bin_str)

	err_bits = use_bits - lengte                  'how many bits are available for error words

	'If the error words and the data words do not take up all the available space then we need to add pre-bits to the bit string
	'It will always be less than 1 word (i.e less than 6, 8, 10, 12 - depending on the selcted layer Aztec code)
	'Pre-bits are excluded from the calculation of error correction words - keep that in mind
	pre_bits = err_bits Mod cap                               'how many pre-bits are required in case there are more bits than what err words take up

	no_of_data_words = Floor(lengte / cap)                    'to be encoded into the mode word
	no_of_error_words = Floor((err_bits - pre_bits) / cap)    'how many full error words will be added

'	Log ("no_of_data_words")
'	Log(no_of_data_words)
'	Log("no_of_error_words")
'	Log(no_of_error_words)
'	Log("Lengte")
'	Log(lengte)
	
	'now get the data words from the data string including bit stuffing and padding but before pre-bits are added (pre-bits to be ignored in error correction calcs)
	Dim data_words(550)  As Int

	teller = 0
	For i = 0 To lengte - 1 Step cap
	  data_words(teller) = convert_to_decimal(sf.Mid(bin_str, i + 1, cap))
	  teller = teller + 1
	Next 
	teller = teller - 1
'   Log ("teller")
'	Log(teller)

	'We now have the following info thus far:
	'1. length = Length of the data bit stream, bit stuffed and padded
	'2. layer = how many layers there will be in the Aztec Code
	'3. siz = the matrix size of the choosen Aztec Code (eg 19 for a 19x19 Aztec Code)
	'4. cap = word size eg 6 bits, 8 bits, 10 bits, 12 bits
	'5. use_bits = total bits in matrix available for data words  and error words
	'6. padd = no of padding bits that were added to the data message
	'7. no_of_data_words = the number of data words determined from the "length" (after stuffing and padding) - bit length of "cap"
	'8. no_of_error_words = the number of error words that will be added to the message (bit length of "cap")
	'9. pre_bits = no of pre-padding bits to be added to the message bit stream (excluded for err word calcs)
	'10. data_words(i) = the (bit stuffed and padded) message bit stream converted into decimal values according to size determined by "cap"

	'Let's now determine the error correction words of the main message
	'First we need to establish the required generator polynomial coefficients
	'We will calculate the coefficients of X and not the Alpha coefficients of the generator polynomial as we will be working with the
	'coefficients of X when calculating the error words to be added to the main message
	'First we need to set up the log / anti-log tables for GF(x)
	'  x = 16 for the mode message with prime polynomial x^4 + x + 1 = decimal 19
	'  x = 64 for layers 1 to 2 with prime polynomial x^6 + x + 1 = decimal 67
	'  x = 256 for layers 3 to 8 with prime polynomial x^8 + x^5 + x^3 + x^2 + 1 = decimal 301
	'  x = 1024 for layers 9 to 22 with prime polynomial x^10 + x^3 + 1 = decimal 1033
	'  x = 4094 for layers 23 to 32 with prime polynomial x^12 + x^6 + x^5 + x^3 + 1 = decimal 4201

	Dim index_of(1025) As Int               'this will suffice for up to layer 11 Aztec codes
	Dim alpha_to(2050) As Int               'but we will double up on this one to make the search easier later on
	Dim gf, PP As Int

	  If layer  = 1 OR layer = 2 Then 
	    gf = 64           'Galios Field for 2^6
	    PP = 67           'Decimal value of Prime Polynomial to be used while setting up log / anti-log tables  (Prime Polynomial = X^6+X+1 = 1000011 = 67)
      End If		
	  If layer >= 3 AND layer <= 8 Then
	    gf = 256          'Galios Field for 2^8
	    PP = 301          'Decimal value of Prime Polynomial to be used while setting up log / anti-log tables  (Prime Polynomial = X^8+X^5+X^3+X^2+1 = 100101101 = 301)
      End If		
	  If layer >= 9 AND layer <= 22 Then
	    gf = 1024         'Galios Field for 2^10
	    PP = 1033         'Decimal value of Prime Polynomial to be used while setting up log / anti-log tables  (Prime Polynomial = X^10+X^3+1 = 10000001001 = 1033)
      End If		
	
	'Set up the log/anti-log table for the required Galios Field (based on the number of layer of the selected Aztec code)
	index_of(0) = 1
	alpha_to(0) = 1
	For i = 1 To gf - 1     'set up log / anti-log tables
	  alpha_to(i) = alpha_to(i - 1) * 2
	  If alpha_to(i) >= gf Then alpha_to(i) = Bit.Xor(alpha_to(i), PP)   'XOR with the decimal value of the prime polynomial
	  index_of(alpha_to(i)) = i                                          'to ensure a finite field
	Next 

	For i = gf To gf * 2 - 1                                             'Double up the log/anti-log table for easy searching of values
	   alpha_to(i) = alpha_to(i - (gf - 1))
	Next 

	index_of(alpha_to(gf - 1)) = gf - 1                                  'Set last missing elements in index_of[]

	'The following code (to generate the generator polynomial with) is credited to Simon Rockliff (University of Adelaide 21 Sep 1989.
	'Translated from C to B4A by Johan Schoeman - April 2014
	'See "www.eccpage.com/rs.c" for original code by Simon Rockliff - see sub "void gen_poly()"
	'Added code to reverse the order of coefficients in array gg[] into array gg_reverse[] - it will be used in this reversed order
	'Discarded the "convert gg[] to index form" part of the code as we will initially be working with
	'the x coefficients of the generator polynomial and not with the alpha coefficients (when calculating the error words)
    Dim nn, kk As Int
	nn = gf - 1
	kk = nn - no_of_error_words       'nn - kk = number of error words required

	Dim gg(550) As Int                'should be at least nn-kk+1
	Dim gg_reverse(550) As Int        'we need to reverse the result that will be in gg[..] after the following code was executed  

	'Let's generate the required Generator Polynomial (in coeffcients of x only). The Message Polynomial (in coeffcients of x only)
	'will be devided by the Generator Polynomial and the remainder will constitute the required error correction words. The calculated
	'error correction words will be appended to message to fina;lise the data that should appear in the Azec code (except for the
	'possibility of having to prepend pre-bits to the entire data stream.

    'Starting alpha values for Aztec codes as per Aztec code specification. It is different for QR Codes. For QR codes gg(0) = 1 and gg(1) = 0
	gg(0) = 2     
	gg(1) = 1

	For i = 2 To nn - kk
	  gg(i) = 1
	    For j = i - 1 To 1 Step -1
	      If gg(j) <> 0 Then
	        gg(j) = Bit.Xor(gg(j - 1), alpha_to((index_of(gg(j)) + i) Mod nn))
	      Else
	        gg(j) = gg(j - 1)
	      End If
	    Next 
	    gg(0) = alpha_to((index_of(gg(0)) + i) Mod nn)
	Next 

	'The coefficients of the Generator Polynomial are now in gg[..] but in reverse order. So, let's reverse the order of the words in gg[..]
	'so that we have the correct order i.e the order in which we want to make use of them.
	For i = 0 To nn - kk
	  gg_reverse(i) = gg(nn - kk - i)
	Next 

	'Now we have the generator polynomial X coefficients in gg_reverse[0.....] -> i.e to calculate the error words from
	'We also have the data words in data_words[0....] and from variable "no_of_data_words" we know how many data words there are
	'Now we can calculate the error correction word for the data stream making use of the X coefficients of the generator polynomial
	'in "gg_reverse[]", the number of error words that are required in "no_of_error_words", and the data words in "data_words[]"

	Dim msg_out(550),coef, gf_mul As Int                             'no of msg_in + gen
	For k = 0 To (no_of_data_words - 1)                              '1-no of msg_in words
	   msg_out(k) = data_words(k)
	Next 
	For i = 0 To no_of_data_words - 1                                '1-no of msg_in words
	   coef = msg_out(i)
	   If coef <> 0 Then
	      For j = 0 To nn - kk                                       '1-no of gen words
	         If gg_reverse(j) = 0 OR coef = 0 Then
	            gf_mul = 0
	         Else
	           gf_mul = alpha_to(index_of(gg_reverse(j)) + index_of(coef))
	         End If
	         msg_out(i + j) = Bit.Xor(msg_out(i + j), gf_mul)
	      Next 
	   End If
	Next 

	'Let's add the data words to msg_out[..]; msg_out[..] already holds the calculated error correction words
	For i = 0 To no_of_data_words - 1                                   '1 less than no of msg in words
	   msg_out(i) = data_words(i)
	Next 
    
	'msg_out[...] now holds the data words and the error correction words
	
	For i = 0 To no_of_data_words + no_of_error_words - 1               'just double checking that we have the count right
	Next
	
	teller = i
	
	' msg_out[...] now contains the decimal values of the data (or message) words and the decimal value of the required error correction words.
	' "teller" now holds the sum of the data words and the error correction words
	' NOW WE NEED TO MAKE A BINARY STRING AGAIN OF ALL THE DATA IN msg_out[..] before adding pre-bits (if required)
	' Just to be sure we will build bin_str from scratch as all words are now in msg_out[..] i.e message and error correction

	bin_str = ""
	For i = 0 To teller - 1
	  bin_str = bin_str & convert_to_binary(msg_out(i), cap)   'convert decimal words to a binary string of length "cap" (i.e 4,5,6,8,10,or 12)
	Next 

	'Now add the pre bits to the bit string
	If pre_bits > 0 Then
	  For i = 1 To pre_bits
	    bin_str = "0" & bin_str                 'add pre-bits of "0" if err words cannot take up full capacity
	  Next 
	End If

	lengte = sf.Len(bin_str)                    'length of data string including bit stuffing, padding, and pre-bits
	'********************************************************************************************************************

	'We will be storing the 1 / 0 's of the entire Aztec code in array mat[..] and then draw the Aztec code from there
	'Let's start building the Aztec code by putting down the finding pattern and the orientation marks
    Dim midpoint As Int
    'First, let's fill mat[...] with 0's
	For i = 0 To 64
	  For j = 0 To 64
	    mat(i, j) = 0
	  Next 
	Next 
    
	midpoint = ((siz - 15) / 2) + 1    'distance in number of modules from outer edge of Aztec code to last module before the mode word
		
    Dim fp() As Int
	'This is the definition of the finding pattern and the orientation marks for a full Aztec Code. It takes up 15 x 15 modules. It includes 
	'the space taken up by the mode message and its error correction words.
	fp = Array As Int(1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, _            'The outmost lines of this matrix also include the space taken
	                  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, _            'up by the mode message and its error correction words
	                  0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, _            'as well as the orientation marks
	                  0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, _
	                  0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, _
	                  0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, _
	                  0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, _
	                  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	           
	'Now let's draw the finding pattern and orientation marks from the above definition/layout
	'We will use "10" to represent a "1" and "9" to represent a "0"
	'This will then make it easier to pack the data stream into the Aztec code as cells containing
	' a 9 and a 10 are occupied and should be skipped during the packing process but then to be interpreted as dark / light modules 
	'during the drawing routine 
	teller = 0
	For i = 1 To 15
	  For j = 1 To 15
	    mat(midpoint + i - 1, midpoint + j - 1) = fp(teller) + 9     'load the data in array 'fp' into the correct place in array 'mat'
	    teller = teller + 1
	  Next 
	Next 
	'The finding pattern and orientation marks are now stored in two dimensional array mat[...]

	'Now we add the timing patterns to mat[..]. We start with the 4 radial timing patterns

	'This will draw the radial timing pattern at the 9 o'clock position
	'We again use 9 and 10 to mark the cells that should contain a 0 or a 1
	Dim wb As Int
	wb = 1  'white / black. We start with a black pixel.
	For j = midpoint - 1 To 1 Step -1                  'midpoint = ((siz - 15) / 2) + 1
	  If wb = 1 Then
	    mat(Floor((siz / 2)) + 1, j) = 10
	    wb = 0
	  Else
	    mat(Floor((siz / 2)) + 1, j) = 9
	    wb = 1
	  End If
	Next 
	  

	'This will draw the radial timing pattern at the 3 o'clock position
	wb = 1
	For j = midpoint + 15 To siz
	  If wb = 1 Then
	    mat(Floor((siz / 2)) + 1, j) = 10
	    wb = 0
	  Else
	    mat(Floor((siz / 2)) + 1, j) = 9
	    wb = 1
	  End If
	Next 
	    
	'This will draw the radial timing pattern at the 12 o'clock position
	wb = 1
	For i = midpoint - 1 To 1 Step -1
	  If wb = 1 Then
	    mat(i, Floor((siz / 2)) + 1) = 10
	    wb = 0
	  Else
	    mat(i, Floor((siz / 2)) + 1) = 9
	    wb = 1
	  End If
	Next 
	  
	'This will draw the radial timing pattern at the 6 o'clock position
	wb = 1
	For i = midpoint + 15 To siz
	  If wb = 1 Then
	    mat(i, Floor((siz / 2)) + 1) = 10
	    wb = 0
	  Else
	    mat(i, Floor((siz / 2)) + 1) = 9
	    wb = 1
	  End If
	Next 


	'Now we need to add the horizontal and vertical timing patterns (if layers > 4)

	If layer > 4 Then
	 'This draws the horizontal timing pattern above the finding pattern, 16 rows from the centre of the finding pattern
	  wb = 1
	  For j = 1 To siz
	    If wb = 1 Then
	      mat(Floor((siz / 2)) - 16 + 1, j) = 10
	      wb = 0
	    Else
	      mat(Floor((siz / 2)) - 16 + 1, j) = 9
	      wb = 1
	    End If
	  Next 

	'This draws the horizontal timing pattern below the finding pattern, 16 rows from the centre of the finding pattern
	  wb = 1
	  For j = 1 To siz
	    If wb = 1 Then
	      mat(Floor((siz / 2)) + 16 + 1, j) = 10
	      wb = 0
	    Else
	      mat(Floor((siz / 2)) + 16 + 1, j) = 9
	      wb = 1
	    End If
	  Next 
	  
	'This draws the vertical timing pattern to the right of the finding pattern, 16 rows from the centre of the finding pattern
	  wb = 1
	  For i = 1 To siz
	    If wb = 1 Then
	      mat(i, Floor((siz / 2)) + 16 + 1) = 10
	      wb = 0
	    Else
	      mat(i, Floor((siz / 2)) + 16 + 1) = 9
	      wb = 1
	    End If
	  Next 
	    
	'This draws the vertical timing pattern to the left of the finding pattern, 16 rows from the centre of the finding pattern
	  wb = 1
	  For i = 1 To siz
	    If wb = 1 Then
	      mat(i, Floor((siz / 2)) - 16 + 1) = 10
	      wb = 0
	    Else
	      mat(i, Floor((siz / 2)) - 16 + 1) = 9
	      wb = 1
	    End If
	  Next

	End If

	'Now we add bin_str to the matrix
	'We will go down, then right, then up, and then left (two rows/columns at a time) and repeat this process
	'until all the bits in the data bit stream have been added. Cells containing a 9 or 10 are already occupied - if they contain a 0 then
	'they are available do accept a bit from the bitstream (a 0 or a 1). If they contain a 9 or 10 they will be skipped in the packing process.
	'We only need to test for a 0 to determine if the cell is available to accept a bit (a 0 or a 1)
	'We start packing from the position 1,1 - the leftmost, top most cell. We pack 1 bit on the left and then one bit to it's immediate right
	'We pack in a counter clockwise direction, from the outside spiraling in towards the core, while skipping cells not containing a 0 
	'(i.e cells containing a 9 or 10) as those cells are already occupied by the core and radial, horizontal, and vertical timing patterns.
	'We will stop packing above the top-left corner of the core
	'Packing during change of direction (down-to-right, right-to-up, up-to-left, left-to-down) needs to be executed correctly
	Dim tr, lc, br, rc, ry, kolom As Int
	tr = 1
	lc = 1
	br = siz
	rc = siz
    
    ry = tr
	kolom = lc
	teller = 1

	Do While teller < sf.Len(bin_str) + 1
	    Do While ry <> br - 1                   'going down
	      If mat(ry, kolom) = 0 Then            'If the cell contains a "0" then the cell is available for packing (i.e it is not 9 or 10)
	        mat(ry, kolom) = sf.Val(sf.mid(bin_str, teller, 1))
	        teller = teller + 1
	        kolom = kolom + 1
	        If kolom = lc + 2 Then
	          kolom = lc
	          ry = ry + 1
	        End If
	      Else
	        ry = ry + 1
	        kolom = lc
	      End If
	      
	    Loop                                     'Stop 2 rows short from the bottom as we need to change direction
	    
	    ry = br
	    kolom = lc
	    
	    Do While kolom <> rc - 1                 'going right
	      If mat(ry, kolom) = 0 Then
	        mat(ry, kolom) = sf.Val(sf.mid(bin_str, teller, 1))
	        teller = teller + 1
	        ry = ry - 1
	        If ry = br - 2 Then
	          ry = br
	          kolom = kolom + 1
	        End If
	      Else
	        kolom = kolom + 1
	        ry = br
	      End If
	      
	    Loop
	    
	    ry = br
	    kolom = rc

	    Do While ry <> tr + 1                    'going up
	      If mat(ry, kolom) = 0 Then
	        mat(ry, kolom) = sf.Val(sf.mid(bin_str, teller, 1))
	        teller = teller + 1
	        kolom = kolom - 1
	        If kolom = rc - 2 Then
	          kolom = rc
	          ry = ry - 1
	        End If
	      Else
	        ry = ry - 1
	        kolom = rc
	      End If
	      
	    Loop 
	    
	    ry = tr
	    kolom = rc

	    Do While kolom <> lc + 1                 'going left
	      If mat(ry, kolom) = 0 Then
	        mat(ry, kolom) = sf.Val(sf.mid(bin_str, teller, 1))
	        teller = teller + 1
	        ry = ry + 1
	        If ry = tr + 2 Then
	          ry = tr
	          kolom = kolom - 1
	        End If
	      Else
	        kolom = kolom - 1
	        ry = tr
	      End If
	      
	    Loop 
	  
	  
	    tr = tr + 2
	    br = br - 2
	    lc = lc + 2
	    rc = rc - 2
	    ry = tr
	    kolom = lc
	    
	    If mat(ry, kolom) <> 0 Then
	      tr = tr + 1
	      br = br - 1
	      lc = lc + 1
	      rc = rc - 1
	      ry = tr
	      kolom = lc
	    End If

	Loop 


    Dim layer_str, word_str, mode_message_str As String 
	'Now we need to add the mode message and its corresponding error words around the core
	'The number of layers are encoded as the actual number of layer minus 1 and converted to a 5-bit string
	layer_str = convert_to_binary(layer - 1, 5)

	'The number of main data words are encoded as the actual number of data words minus 1 and converted to an 11-bit string
	word_str = convert_to_binary(no_of_data_words - 1, 11)

	'Assemble the binary mode message from "layer_str" and "word_str" - it is a string of 11 chars/bits
	mode_message_str = layer_str & word_str


	Dim mode_mes(4) As Int                    'For full Aztec code the mode message will consist of 4 x 4-bit words i.e 4 x decimal numbers
	teller = 0
	For i = 1 To sf.Len(mode_message_str) Step 4
	  a = sf.mid(mode_message_str, i, 4)
	  mode_mes(teller) = convert_to_decimal(a)   'Get the decimal value of each of the 4 x 4-bit mode message words
	  teller = teller + 1
	Next 

	'Now mode_message_str is a "16 bit" string representing the mode message in binary form
	'It is constructed from the number of layers (5 bits) and the number of data words (11 bits)
	'The number of layers is encoded as 1 less than the actual number of layers (i.e the count starts at 0)
	'The number of data words are encoded as 1 less that the actual number of data words (i.e the count starts at 0)
	'This 16 bits represents 4 x 4-bit words that have been converted to 4 x decimal numbers and are now stored in mode_mes[..]
	'The 4 x decimal numbers will be used to calculate the mode message error word with
	'There should be 40 mode bits around. It is made up of the 4 x 4-bit mode message and 6 x 4-bit error words

	'Now let's determine the 6 x error correction words
	'First we need to set up a log / antilog table in GF(16) with a prime polynomial representing decimal value 19

	gf = 16     'Galios field of 2^4
	PP = 19     'The decimal representation of the Prime Polynomial that should be used to create the log / anti-log tables with
	            'thereby ensuring a finite arithmetic field

	index_of(0) = 1
	alpha_to(0) = 1
	For i = 1 To gf - 1     'set up log / anti-log tables
	  alpha_to(i) = alpha_to(i - 1) * 2
	  If alpha_to(i) >= gf Then alpha_to(i) = Bit.Xor(alpha_to(i),PP)
	  index_of(alpha_to(i)) = i
	Next 

	For i = gf To gf * 2 - 1
	   alpha_to(i) = alpha_to(i - (gf - 1))
	Next 

	index_of(alpha_to(gf - 1)) = gf - 1 ' Set last missing elements in index_of[]

	'We need a generator polynomial the we can devide the mode message by to get the error words
	'For a full Aztec code there will alway be 6 error correction words that need to be appended to the mode data words
	'The x-coefficients of the generator polynomial for calculating the mode message error words will always be 1,7,9,3,12,10,12 for full Aztec codes
	'So, no need to write code to generate the mode message generator polynomial
	'It is the same principle as what we have used for the main data except that this should be in GF(16) and will always require only 6 error words

	'We now have the mode message data words, the x coefficients of the mode message generator polynomial, and the log/anti-log table
	'Now we can calculate the error correction words for the mode message

    Dim gen() As Int
	gen = Array As Int(1, 7, 9, 3, 12, 10, 12)
	no_of_data_words = 4

	Dim msg_out_2(11) As Int                                         'no of msg_in + gen
	For k = 0 To (no_of_data_words - 1)                              'no of msg_in words - 1
	   msg_out_2(k) = mode_mes(k)
	Next 
	For i = 0 To no_of_data_words - 1                                'no of msg_in words - 1
	   coef = msg_out_2(i)
	   If coef <> 0 Then
	      For j = 0 To 6                                             'no of gen words - 1
	         If gen(j) = 0 OR coef = 0 Then
	            gf_mul = 0
	         Else
	           gf_mul = alpha_to(index_of(gen(j)) + index_of(coef))
	         End If
	         msg_out_2(i + j) = Bit.Xor(msg_out_2(i + j), gf_mul)
	      Next 
	   End If
	Next 

	'Let's add the data words to msg_out[..]; msg_out[..] already holds the calculated error correction words
	For i = 0 To no_of_data_words - 1                                   'no of msg_in words - 1
	   msg_out_2(i) = mode_mes(i)
	Next 

	'We now have the 10 x decimal values representing the mode message and mode message error words in msg_out_2[0...9]
	'msg_out_2[0..3] is the 4 message words and msg_out_2[4..9] is the 6 error correction words
	'Now we need to convert each of them to a 4-bit binary string and go place them in their correct positions around the core
	'The packing of the mode words and error words start at the top left of the finding pattern and goes clock wise around the 
	'core - skipping the bit in line with the radial timing pattern i.e 5 bits just before and 5 bits just after the centre of the radial timing pattern
	a = ""
	For i = 0 To 9
	  a = a & convert_to_binary(msg_out_2(i), 4)
	Next 
    
	Dim xcor, ycor As Int
	xcor = ((siz - 15) / 2) + 1             'going right
	ycor = ((siz - 15) / 2) + 3
	teller = 1
	For i = 1 To 10
	  mat(xcor, ycor) = sf.mid(a, i, 1)
	  ycor = ycor + 1
	  teller = teller + 1
	  If teller = 6 Then
	    ycor = ycor + 1
	  End If
	Next 

	xcor = ((siz - 15) / 2) + 3              'going down
	ycor = ((siz - 15) / 2) + 15
	teller = 1
	For i = 11 To 20
	  mat(xcor, ycor) = sf.mid(a, i, 1)
	  xcor = xcor + 1
	  teller = teller + 1
	  If teller = 6 Then
	    xcor = xcor + 1
	  End If
	Next 

	xcor = ((siz - 15) / 2) + 15             'going left
	ycor = ((siz - 15) / 2) + 13
	teller = 1
	For i = 21 To 30
	  mat(xcor, ycor) = sf.mid(a, i, 1)
	  ycor = ycor - 1
	  teller = teller + 1
	  If teller = 6 Then
	    ycor = ycor - 1
	  End If
	Next 
	  
	xcor = ((siz - 15) / 2) + 13              'going up
	ycor = ((siz - 15) / 2) + 1
	teller = 1
	For i = 31 To 40
	  mat(xcor, ycor) = sf.mid(a, i, 1)
	  xcor = xcor - 1
	  teller = teller + 1
	  If teller = 6 Then
	    xcor = xcor - 1
	  End If
	Next 
    
	'We now have all the required infor to draw the Aztec code in mat[...]
	'Now go draw the Aztec Code with the info in mat[..] with "Sub teken" --->> "teken" means "draw"
'	EditText2.Visible = False
'    EditText2.Text = no_of_data_words & " " & no_of_error_words & " " & stuff_tel & " " & padd & " " & siz
    teken
Else
  ToastMessageShow ("Input is larger than what layer 11 can handle", True)	
End If

End Sub
'We bring the integer number to be converted and the required length of the binary string to be returned to this subroutine
'The output is a binary string of the required length
Private Sub convert_to_binary(getal As Int, places As Int) As String

Dim rev_len_dat_bin, len_dat_bin,getal1 As String
Dim e,f,g,plekke As Int

rev_len_dat_bin = ""
e = getal
plekke = places
If e <> 0 Then
	Do While e <> 0
	  f = Floor(e / 2)
	  g = e - (f * 2)
	  If g = 1 Then rev_len_dat_bin = rev_len_dat_bin & "1"
	  If g = 0 Then rev_len_dat_bin = rev_len_dat_bin & "0"
	  e = f
	Loop
	'reverse the binary string to get the order right
	len_dat_bin = ""
	For x = 1 To sf.len(rev_len_dat_bin)
	  len_dat_bin = sf.Mid(rev_len_dat_bin, x, 1) & len_dat_bin
	Next

	rev_len_dat_bin = sf.Trim(rev_len_dat_bin)
	len_dat_bin = sf.Trim(len_dat_bin)

	getal1 = len_dat_bin
	If sf.Len(getal1) < plekke Then
      For i = sf.Len(getal1) To plekke - 1
        getal1 = "0" & getal1
      Next	
	End If  
	Return getal1                    'this is the binary string with the required length
Else
  getal1 = ""
  For i = 1 To plekke
    getal1 = "0" & getal1            'this is the binary string with the required length but all 0's
  Next	
  Return getal1

End If


End Sub
'Take a binary string and convert it to a decimal number
Private Sub convert_to_decimal(b As String) As Int

Dim waarde As Int
waarde = 0    'convert binary to decimal
For i = sf.Len(b) To 1 Step -1
 If sf.Mid(b, i, 1) = 1 Then
   waarde = waarde + Power(2, (sf.Len(b) - i))     'bit value from LSB to MSB ^2 and add them together
 End If
Next 

Return waarde

End Sub

'Store the Aztec code as a PNG file in the pictures folder (File.DirRootExternal, "pictures/Aztec.png")
'and load it into ImageView1 
Private Sub teken

Dim b As Bitmap
Dim p1 As Path
Dim siz1 As Int
siz1 = 8        'was 9 27 May 2014
b.InitializeMutable(siz*siz1+50dip,siz*siz1+50dip)      'This will set a border of 50dip around the image
                                                        'siz is the size of the matrix/Aztec code eg 37 x 37 modules  
Dim c As Canvas
Dim x As Int
Dim y As Int

c.initialize2(b)
Rect1.Initialize (0,0,0,0)
Rect1.Left = 0                                          'Set the size of the drawing area and fill it with WHITE
Rect1.Right = siz * siz1 + 50dip        
Rect1.Top = 0            
Rect1.Bottom = siz * siz1 + 50dip         

'c.DrawRect(Rect1,bcolor,True,0)

'***********************************************************************
			Dim gd As GradientDrawable
			Dim colours(4) As Int
			colours(0) = Colors.ARGB(255,66,108,188)
			colours(1) = Colors.ARGB(255,212,42,118)
			colours(2) = Colors.ARGB(255,52,84,202)
			colours(3) = Colors.ARGB(255,55,230,24)			
			gd.Initialize("TR_BL",colours)

			Dim r As Reflector
		    Dim style As Int
			style = Rnd(0,3)
			
			r.Target = gd
'			r.runmethod4("setColors",Array As Object(Array As Int(0xFFFF00FF,0xFFFF4500,0xFF9932CC,0xFF6495ED)), Array As String("[I"))
			r.RunMethod2("setGradientType",style,"java.lang.int")                          '0 = linear, 1 = radial, 2 = sweep/angled/diamond
			r.RunMethod2("setGradientRadius",(siz*siz1+50dip)/1.5,"java.lang.float")
            r.RunMethod2("setCornerRadius",20,"java.lang.float")
'            r.RunMethod4("setCornerRadii", Array As Object(Array As Float(20,20,20,20,20,20,20,20)), Array As String("[F"))

			c.DrawDrawable(gd,Rect1)
'***********************************************************************

x = Floor (((siz * siz1 + 50dip) - (siz * siz1))/2)     'Find the centre points and step back to make it appear centre in the PNG file 
y = Floor (((siz * siz1 + 50dip) - (siz * siz1))/2)
         

For i =  1 To siz                                      'siz = the size of the Aztec code e.g 37 x 37 modules
  For j = 1 To siz
    If mat(i,j) = 10  OR mat(i,j) = 1 Then             'If 10 or 1 then draw a black module
		Rect1.Left = x       
		Rect1.Right = x + siz1            
		Rect1.Top = y            
		Rect1.Bottom = y + siz1
'		Rect1.Left = x + 1      
'		Rect1.Right = x + siz1 - 2           
'		Rect1.Top = y + 1            
'		Rect1.Bottom = y + siz1 - 2	
		If shp = "c" OR shp = "C" Then
          c.Drawcircle(x+siz1/2,y+siz1/2,siz1/2,fcolor,True,0)        'draw circles
		End If
		If shp = "s" OR shp = "S" Then
          c.DrawRect(Rect1, fcolor, True, siz1)                  'draw squares
		End If  
		If shp = "d" OR shp = "D" Then
          c.DrawRectRotated(Rect1, fcolor, True, siz1,45)        'draw diamonds
		End If  
		If shp = "o" OR shp = "O" Then
          c.DrawOval(Rect1, fcolor, True, siz1)                  'draw ovals
'		  Log("o")
		End If  
		If shp = "td" OR shp = "Td" OR shp = "TD" OR shp = "tD" Then   'draw triangle pointing down
		  p1.Initialize(x,y)
		  p1.LineTo(x+siz1/2,y+siz1)
		  p1.LineTo(x+siz1,y)
		  c.DrawPath(p1,fcolor,True,0)
		End If
		If shp = "tu" OR shp = "Tu" OR shp = "TU" OR shp = "tU" Then   'draw triangle pointing up
		  p1.Initialize(x+siz1/2,y)
		  p1.LineTo(x,y+siz1)
		  p1.LineTo(x+siz1,y+siz1)
		  c.DrawPath(p1,fcolor,True,0)
		End If		
		If shp = "tl" OR shp = "Tl" OR shp = "TL" OR shp = "tL" Then   'draw triangle pointing left
		  p1.Initialize(x+siz1,y)
		  p1.LineTo(x+siz1,y+siz1)
		  p1.LineTo(x,y+siz1/2)
		  c.DrawPath(p1,fcolor,True,0)
		End If			
		If shp = "tr" OR shp = "Tr" OR shp = "TR" OR shp = "tR" Then   'draw triangle pointing right
		  p1.Initialize(x,y)
		  p1.LineTo(x,y+siz1)
		  p1.LineTo(x+siz1,y+siz1/2)
		  c.DrawPath(p1,fcolor,True,0)
		End If							
		If shp = "a" OR shp = "A" Then                           'draw asterix
		  p1.Initialize(x+siz1/2,y+siz1/2)
		  p1.LineTo(x,y)
		  p1.LineTo(x+siz1,y)
		  p1.LineTo(x+siz1,y+siz1)	
		  p1.LineTo(x,y+siz1)			  
		  p1.LineTo(x+siz1/2,y)			  
		  p1.LineTo(x+siz1/2,y+siz1)			  
		  p1.LineTo(x,y+siz1/2)	
		  p1.LineTo(x+siz1,y+siz1)	
		  c.DrawPath(p1,fcolor,True,0)
		End If		
		If sf.Left(shp,2) = "tx" OR sf.Left(shp,2) = "Tx" OR sf.Left(shp,2) = "TX" OR sf.Left(shp,2) = "tX" Then   'draw using text
          c.DrawText(sf.Right(shp,1),x,y,Typeface.DEFAULT_BOLD,12,fcolor,"CENTER")                      
		End If				
		
		
	Else If mat(i,j) = 9 OR mat(i,j) = 0 Then Then     'If 9 or 0 then draw a white module
		Rect1.Left = x       
		Rect1.Right = x + siz1            
		Rect1.Top = y            
		Rect1.Bottom = y + siz1
'		Rect1.Left = x + 1      
'		Rect1.Right = x + siz1 - 2           
'		Rect1.Top = y + 1            
'		Rect1.Bottom = y + siz1 - 2	   	 

'		If shp = "c" OR shp = "C" Then
'          c.Drawcircle(x+siz1/2,y+siz1/2,siz1/2,bcolor,True,0)        'draw circles
'		End If
'		If shp = "s" OR shp = "S" Then
'         c.DrawRect(Rect1, bcolor, True, siz1)                  'draw squares
'		End If  
'		If shp = "d" OR shp = "D" Then
'          c.DrawRectRotated(Rect1, bcolor, True, siz1,45)       'draw diamonds
'		End If  				
'		If shp = "o" OR shp = "O" Then
'         c.DrawOval(Rect1, bcolor, True, siz1)                  'draw ovals
'		  Log("o")
'		End If  					
	Else                              'only if there is an error in the code that populates the Aztec code matrix i.e in array "mat[i,j]"
		Rect1.Left = x                'this "Else" statement is pupely for debugging purposes - making sure "mat[i,j]" had been packed correctly  
		Rect1.Right = x + siz1            
		Rect1.Top = y            
		Rect1.Bottom = y + siz1            	 
		c.DrawRect(Rect1, Colors.Red, True, siz1)
	End If	
	x = x + siz1
  Next
    x = Floor (((siz * siz1 + 50dip) - (siz * siz1))/2)      
	y = y + siz1                                             
Next 

'See Erel's posting at http://www.basic4ppc.com/android/forum/threads/saving-a-picture.10200/#post-56686 for detail of the code below
'Marginally changed to suite my requirements

Dim out As OutputStream
out = File.OpenOutput(File.DirRootExternal, "/pictures/Aztec.png", False)
b.WriteToStream(out, 100, "PNG")                                           'Export the generated bitmap to a PNG and save in in the pictures folder
out.flush                                                                   'Make sure nothing is stuck that should have been written
out.Close

'send the intent that asks the media scanner to scan the file
Dim a123 As Intent
a123.Initialize("android.intent.action.MEDIA_SCANNER_SCAN_FILE", _
    "file://" & File.Combine(File.DirRootExternal, "/pictures/Aztec.png"))   '...so that the file is visible....    
Dim p As Phone
p.SendBroadcastIntent(a123)



End Sub